/*****************************************************************************
 * mixer.c: some functions to control a sound mixer
 *****************************************************************************
 * $Id: mixer.c,v 1.4 2003/09/07 14:04:57 alainjj Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * Original code:
 *
 *  (c) 1998 Gerd Knorr <kraxel@cs.tu-berlin.de>
 *
 *****************************************************************************/
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>

#include "config.h"

#include "mixer.h"

extern int debug;


static char *names[] = SOUND_DEVICE_NAMES;

enum {
  LINE=0,
  FRONT,
  REAR,
  IGAIN,
  REARCONTROL,
  NB_DEVICES
};
	

static char *device_list[NB_DEVICES] = { (char *) NULL, "pcm", "dig1", "igain", (char *)NULL };
#ifdef HAVE_ALSA05
  static char *alsa_group_list[NB_DEVICES] = { (char *) NULL, "PCM", "Rear", "Input Gain", (char *)NULL };
#endif
#ifdef HAVE_ALSA09
  static char *alsa_group_list[NB_DEVICES] = { (char *) NULL, "PCM", "Wave Surround", "Capture", "SB Live Analog/Digital Output Jack" };
#endif
static int mix;
static int dev_id[NB_DEVICES] = { -1, -1, -1, -1 };
static int volume[NB_DEVICES];    /* 0x6363 <=> 99 on left channel and 99 on right channel */
static int muted[NB_DEVICES];
static int captured[NB_DEVICES];
static int devmask, recmask, old_recsrc, recsrc;
static int cur_dev = -1;


int
mixer_open (char *filename, char *device)
{
  int i, j;
#ifdef HAVE_ALSA
  char command[64];
#endif
  int dix = 0x0a0a;
  
  device_list[0] = (char *) malloc (strlen (device) + 1);
  strcpy (device_list[0], device);

#ifdef HAVE_ALSA
  alsa_group_list[0] = (char *) malloc (strlen (device) + 1);
  if (strcasecmp(device, "line")==0)
    strcpy (alsa_group_list[0], "Line");
  else
    strcpy (alsa_group_list[0], device);
#endif

  if (-1 == (mix = open (filename, O_RDONLY)))
    {
      perror ("mixer open");
      return -1;
    }

  if (-1 == ioctl (mix, MIXER_READ (SOUND_MIXER_DEVMASK), &devmask))
    {
      perror ("mixer read devmask");
      return -1;
    }
  //printf("devmask = 0x%04X\n", devmask);    

  if (-1 == ioctl (mix, MIXER_READ (SOUND_MIXER_RECMASK), &recmask))
    {
      perror ("mixer read recmask");
      return -1;
    }
  //printf("recmask = 0x%04X\n", recmask);    

  for (j = 0; j < NB_DEVICES; j++)
    {
      if (device_list[j] == (char *)NULL)
        continue;

      for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
        {
          if (debug)
            printf("mixer names [%d] = '%s'\n", i, names[i]);

          if ((1 << i) & devmask
              && strcasecmp (names[i], device_list[j]) == 0)
            {
              if (-1 == ioctl (mix, MIXER_READ (i), &volume[j]))
                {
                  perror ("mixer read volume");
                  return -1;
                }
              else
                {
                  dev_id[j] = i;
                  muted[j] = 0;
                  if (volume[j] == 0)
                    {
                      if (j == IGAIN)
                        volume[j] = dix;
                      else
                        volume[j] = 0x4444;
                      if (-1 == ioctl (mix, MIXER_WRITE (i), &volume[j]))
                        {
                          perror ("mixer write volume");
                          return -1;
                        }
                    }

                  if (debug)
                    printf
                      ("mixer: %s (%d) opened with volume = %d = 0x%04X\n",
                       device_list[j], dev_id[j], volume[j], volume[j]);
                }
              break;
            }
        }
    }



  if (-1 == dev_id[0])
    {
      fprintf (stderr, "mixer: hav'nt found device '%s'\nmixer: available: ",
               device);
      for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
        if ((1 << i) & devmask)
          fprintf (stderr, " '%s'", names[i]);
      fprintf (stderr, "\n");
    }

  if (-1 == ioctl (mix, MIXER_READ (SOUND_MIXER_RECSRC), &recsrc))
    {
      perror ("mixer read recsrc");
      return -1;
    }
  old_recsrc = recsrc;
  for (i = 0; i < NB_DEVICES; i++)
    {
      captured[i] = 0;
      if ((1 << dev_id[i]) & recmask)
        {
          if ((1 << dev_id[i]) & recsrc)
            captured[i] = 1;
        }
      if (debug)
        printf ("mixer: mixer_open: captured (%s) = %d\n", device_list[i],
                captured[i]);
    }

  // Make sure IGAIN isn't muted
#if defined(HAVE_ALSA)
  sprintf(command, "amixer set '%s',0 unmute >/dev/null", alsa_group_list[IGAIN]);
  //printf("%s\n", command);
  system(command);
  if (debug)
    fprintf(stderr, "mixer: mixer_open: ALSA forcing IGAIN unmute\n");
#else
  if (-1 == ioctl (mix, MIXER_READ (dev_id[IGAIN]), &volume[IGAIN]))
    fprintf(stderr, "mixer: mixer_open: couldn't read %s volume\n", device_list[IGAIN]);

  if(debug)
    fprintf(stderr, "****************igain volume = 0x%04X\n", volume[IGAIN]);

  if ( ((volume[IGAIN] & 0xFF) == 0) || ((volume[IGAIN] & 0xFF00)>>8 == 0) )
    {
      if (-1 == ioctl (mix, MIXER_WRITE (dev_id[cur_dev]), &dix))
        fprintf(stderr, "mixer: mixer_open: couldn't write %s volume\n", device_list[IGAIN]);
  if (debug)
    fprintf(stderr, "mixer: mixer_open: OSS setting IGAIN volume to %04X\n", dix);
    }
#endif

  cur_dev = 0;

  return (-1 != dev_id[0]) ? 0 : -1;
}

void
mixer_close (void)
{
  int i;

  if (debug)
    fprintf (stderr, "mixer: close\n");

  close (mix);

  for (i = 0; i < NB_DEVICES; i++)
    dev_id[i] = -1;
  cur_dev = -1;

  if (device_list[0])
    {
      free (device_list[0]);
      device_list[0] = (char *) NULL;
    }
}

int
mixer_get_volume (void)
{
  //printf("mixer: mixer_get_volume: volume (%s) = 0x%02X\n", device_list[cur_dev], volume[cur_dev]&0x7f);
  return (-1 == dev_id[cur_dev]) ? -1 : (volume[cur_dev] & 0x7f);
}

int
mixer_set_volume (int val)
{
  if (-1 == dev_id[cur_dev])
    return -1;

  //printf("mixer: mixer_set_volume: val = %d = 0x%02X\n", val, val);
  val &= 0x7f;
  volume[cur_dev] = val | (val << 8);;
  //printf("mixer: mixer_set_volume: volume (%s) = %d = 0x%04X\n", device_list[cur_dev],volume[cur_dev], volume[cur_dev]);

  if (-1 == ioctl (mix, MIXER_WRITE (dev_id[cur_dev]), &volume[cur_dev]))
    {
      perror ("mixer write volume");
      return -1;
    }

  muted[cur_dev] = 0;
  return 0;
}

int
mixer_mute (void)
{
#ifdef HAVE_ALSA
  char command[64];
#else
  int zero = 0;
#endif
  
  if (-1 == dev_id[cur_dev])
    return -1;

#ifdef HAVE_ALSA
  sprintf(command, "amixer set '%s',0 mute >/dev/null", alsa_group_list[cur_dev]);
  system(command);
#else
  if (-1 == ioctl (mix, MIXER_WRITE (dev_id[cur_dev]), &zero))
    return -1;
#endif

  muted[cur_dev] = 1;

  return 0;
}

int
mixer_unmute (void)
{
#ifdef HAVE_ALSA
  char command[64];
#endif

  if (-1 == dev_id[cur_dev])
    return -1;

#ifdef HAVE_ALSA
  sprintf(command, "amixer set '%s',0 unmute >/dev/null", alsa_group_list[cur_dev]);
  system(command);
#else
  if (-1 == ioctl (mix, MIXER_WRITE (dev_id[cur_dev]), &volume[cur_dev]))
    return -1;
#endif

  muted[cur_dev] = 0;

  return 0;
}

int
mixer_get_muted (void)
{
  return (-1 == dev_id[cur_dev]) ? -1 : muted[cur_dev];
}

int
mixer_capture (void)
{
  if (-1 == dev_id[cur_dev])
    return -1;

  if (!((1 << dev_id[cur_dev]) & recmask))
    return -1;

  recsrc = (1 << dev_id[cur_dev]);
  if (-1 == ioctl (mix, MIXER_WRITE (SOUND_MIXER_RECSRC), &recsrc))
    return -1;

  captured[cur_dev] = 1;

  return 0;
}


int
mixer_uncapture (void)
{
  if (-1 == dev_id[cur_dev])
    return -1;

  if (!((1 << dev_id[cur_dev]) & recmask))
    return -1;

  recsrc = old_recsrc;
  if (-1 == ioctl (mix, MIXER_WRITE (SOUND_MIXER_RECSRC), &recsrc))
    return -1;

  captured[cur_dev] = 0;

  return 0;
}

int
mixer_get_captured (void)
{
  return (-1 == dev_id[cur_dev]) ? -1 : captured[cur_dev];
}

int
mixer_toggle_device (void)
{
  cur_dev++;
  cur_dev %= NB_DEVICES;
  return 0;
}

int
mixer_set_device (int dev)
{
  if ((dev < 0) || (dev > NB_DEVICES - 1))
    return -1;

  if (dev_id[dev] == -1)
    return -1;

  cur_dev = dev;

  return 0;
}
